==> base-1.1.28.tar.gz: base/src/Net.cc <==
///////////////////////////////////////////////////////////////////////////////
//                                                         
// Net.cc
// ------
// Base network driver
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2007 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: Net
// 
// Description: All operations on network resources
//
// Status: CLEAN
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _REENTRANT
#define _REENTRANT    /* basic 3-lines for threads */
#endif

#include <unistd.h>

#ifdef HAVE_MINGW32
#include <windows.h>
#include <winsock2.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif
#include <errno.h>

#include "Exception.h"
#include "Net.h"


Net::Net(int msgBufSize)
{

#ifdef HAVE_MINGW32

    WSADATA wsa;
    long rc;
    rc = WSAStartup(MAKEWORD(2,0), &wsa);

#endif

    _msgBufSize = msgBufSize;
    _csock=0;
}

Net::~Net()
{
    if ( _csock )
    {
	::close(_csock);
    }
}

NetHandler* Net::connect(const Chain& hostname, int port)
{

    struct sockaddr_in sa;
    struct hostent     *he;
    
    if ((he = gethostbyname((char*)hostname)) == NULL)
    {
	Chain msg = Chain("Cannot resolve hostname ") + hostname;
	throw Exception(EXLOC, msg);
    }    
    int socket;
    if ( (socket = ::socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
    {
	Chain msg = Chain("socket system error : ") + Chain(strerror(errno)); 
    	throw Exception(EXLOC, msg);
    }

    memcpy(&sa.sin_addr, he->h_addr_list[0], he->h_length);
    sa.sin_family = AF_INET;
    sa.sin_port = htons(port);

    if (::connect(socket, (struct sockaddr *)&sa, sizeof(sa)) < 0 )
    {
	Chain msg = Chain("connect system error : ") + Chain(strerror(errno)); 
    	throw Exception(EXLOC, msg);
    }

    NetHandler* pNH = new NetHandler(socket, _msgBufSize);
    
    return pNH;
}

void Net::serve(int port)
{

    struct sockaddr_in sa;
    int addrlen;

    if ( (_csock=socket(AF_INET,SOCK_STREAM,0)) == 0)
    	throw Exception(EXLOC, "socket system error");


    int optVal = 1;
    if ( setsockopt(_csock, SOL_SOCKET, SO_REUSEADDR, &optVal, sizeof(int)) )
    {
	Chain msg = Chain("setsockopt system error : ") + Chain(strerror(errno));
	throw Exception(EXLOC, msg);
    }
    
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = INADDR_ANY;
    sa.sin_port = htons(port);

    if ( bind(_csock,(struct sockaddr *)&sa, sizeof(sa)) < 0)
    {
	Chain msg = Chain("bind system error on port ") + Chain(port) + Chain(" : ") + Chain(strerror(errno));
    	throw Exception(EXLOC, msg);
    }

    if (listen(_csock,3) < 0 )
    {
	Chain msg = Chain("listen system error : ") + Chain(strerror(errno)); 
    	throw Exception(EXLOC, msg);
    }
}

/* with nextRequest, incoming requests are accepted on a well known port. Further
   incoming messages from the partner are handled outside the method ( in a dedicated thread ) 
 */

NetHandler* Net::nextRequest(int timeout)
{
   
    int s;
    struct timeval t;
    t.tv_sec = timeout / 1000000;
    t.tv_usec = timeout % 1000000;

    fd_set fdSet;
    
    FD_ZERO(&fdSet);
    FD_SET(_csock, &fdSet);
    
    if ( (s = ::select(_csock + 1, &fdSet, NULL, NULL, &t)) < 0 )
    {
	Chain msg = Chain("select system error : ") + Chain(strerror(errno)); 
	throw Exception(EXLOC, msg);
    }

    int nsocket = 0;
    
    if ( s > 0 )
    {
	if ( FD_ISSET(_csock, &fdSet) )
	{
	    
	    // cout << "New connecction req detected .." << endl;
	    FD_CLR(_csock, &fdSet);
	    
	    struct sockaddr_in sa;

#ifdef HAVE_MINGW32
	    int sa_size=sizeof(sa);
#else
	    socklen_t sa_size=sizeof(sa);
#endif
	    
	    // check for new connections ( non-blocking )
	    if ( (nsocket = ::accept(_csock, (struct sockaddr *)&sa, &sa_size)) < 0)
	    {

#ifdef HAVE_MINGW32
		if ( errno != 0 )
		{
		    Chain msg = Chain("accept system error: ") + Chain(strerror(errno));
		    throw Exception(EXLOC, msg);
		}
#else
		if ( errno != EWOULDBLOCK )
		{
		    Chain msg = Chain("accept system error: ") + Chain(strerror(errno));
		    throw Exception(EXLOC, msg);
		}
#endif
	    }
	    if ( nsocket > 0 )
	    {
		// cout << "Accepting new connecction ..." << nsocket  << endl;
		NetHandler* pNetHandle = new NetHandler(nsocket, _msgBufSize);

		try 
		{
		    pNetHandle->readMsg();
		}
		catch ( Exception e)
		{
		    
		    pNetHandle->disconnect();
		    delete pNetHandle;
		    
		    return 0;
		}
		return pNetHandle;
			
	    }				
	}
    
    }
    return 0;
}

==> base-1.1.31.tar.gz: base/src/Net.cc <==
///////////////////////////////////////////////////////////////////////////////
//                                                         
// Net.cc
// ------
// Base network driver
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2007 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: Net
// 
// Description: All operations on network resources
//
// Status: CLEAN
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _REENTRANT
#define _REENTRANT    /* basic 3-lines for threads */
#endif

#include <unistd.h>

#ifdef HAVE_MINGW32
#include <windows.h>
#include <winsock2.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif
#include <errno.h>

#include "Exception.h"
#include "Net.h"


Net::Net(int msgBufSize, int sizeBufLen)
{

#ifdef HAVE_MINGW32

    WSADATA wsa;
    long rc;
    rc = WSAStartup(MAKEWORD(2,0), &wsa);

#endif

    _msgBufSize = msgBufSize;
    _sizeBufLen = sizeBufLen;
    _csock=0;
}

Net::~Net()
{
    if ( _csock )
    {
	::close(_csock);
    }
}

NetHandler* Net::connect(const Chain& hostname, int port)
{

    struct sockaddr_in sa;
    struct hostent     *he;
    
    if ((he = gethostbyname((char*)hostname)) == NULL)
    {
	Chain msg = Chain("Cannot resolve hostname ") + hostname;
	throw Exception(EXLOC, msg);
    }    
    int socket;
    if ( (socket = ::socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
    {
	Chain msg = Chain("socket system error : ") + Chain(strerror(errno)); 
    	throw Exception(EXLOC, msg);
    }

    memcpy(&sa.sin_addr, he->h_addr_list[0], he->h_length);
    sa.sin_family = AF_INET;
    sa.sin_port = htons(port);

    if (::connect(socket, (struct sockaddr *)&sa, sizeof(sa)) < 0 )
    {
	Chain msg = Chain("connect system error : ") + Chain(strerror(errno)); 
    	throw Exception(EXLOC, msg);
    }

    NetHandler* pNH = new NetHandler(socket, _msgBufSize, _sizeBufLen);
    
    return pNH;
}

void Net::serve(int port)
{

    struct sockaddr_in sa;
    int addrlen;

    if ( (_csock=socket(AF_INET,SOCK_STREAM,0)) == 0)
    	throw Exception(EXLOC, "socket system error");


    int optVal = 1;
    if ( setsockopt(_csock, SOL_SOCKET, SO_REUSEADDR, &optVal, sizeof(int)) )
    {
	Chain msg = Chain("setsockopt system error : ") + Chain(strerror(errno));
	throw Exception(EXLOC, msg);
    }
    
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = INADDR_ANY;
    sa.sin_port = htons(port);

    if ( bind(_csock,(struct sockaddr *)&sa, sizeof(sa)) < 0)
    {
	Chain msg = Chain("bind system error on port ") + Chain(port) + Chain(" : ") + Chain(strerror(errno));
    	throw Exception(EXLOC, msg);
    }

    if (listen(_csock,3) < 0 )
    {
	Chain msg = Chain("listen system error : ") + Chain(strerror(errno)); 
    	throw Exception(EXLOC, msg);
    }
}

/* with nextRequest, incoming requests are accepted on a well known port. Further
   incoming messages from the partner are handled outside the method ( in a dedicated thread ) 
 */

NetHandler* Net::nextRequest(int timeout)
{
   
    int s;
    struct timeval t;
    t.tv_sec = timeout / 1000000;
    t.tv_usec = timeout % 1000000;

    fd_set fdSet;
    
    FD_ZERO(&fdSet);
    FD_SET(_csock, &fdSet);
    
    if ( (s = ::select(_csock + 1, &fdSet, NULL, NULL, &t)) < 0 )
    {
	Chain msg = Chain("select system error : ") + Chain(strerror(errno)); 
	throw Exception(EXLOC, msg);
    }

    int nsocket = 0;
    
    if ( s > 0 )
    {
	if ( FD_ISSET(_csock, &fdSet) )
	{
	    
	    // cout << "New connecction req detected .." << endl;
	    FD_CLR(_csock, &fdSet);
	    
	    struct sockaddr_in sa;

#ifdef HAVE_MINGW32
	    int sa_size=sizeof(sa);
#else
	    socklen_t sa_size=sizeof(sa);
#endif
	    
	    // check for new connections ( non-blocking )
	    if ( (nsocket = ::accept(_csock, (struct sockaddr *)&sa, &sa_size)) < 0)
	    {

#ifdef HAVE_MINGW32
		if ( errno != 0 )
		{
		    Chain msg = Chain("accept system error: ") + Chain(strerror(errno));
		    throw Exception(EXLOC, msg);
		}
#else
		if ( errno != EWOULDBLOCK )
		{
		    Chain msg = Chain("accept system error: ") + Chain(strerror(errno));
		    throw Exception(EXLOC, msg);
		}
#endif
	    }
	    if ( nsocket > 0 )
	    {
		// cout << "Accepting new connecction ..." << nsocket  << endl;
		NetHandler* pNetHandle = new NetHandler(nsocket, _msgBufSize, _sizeBufLen);

		try 
		{
		    pNetHandle->readMsg();
		}
		catch ( Exception e)
		{
		    
		    pNetHandle->disconnect();
		    delete pNetHandle;
		    
		    return 0;
		}
		return pNetHandle;
			
	    }				
	}
    
    }
    return 0;
}
